#include <iostream>
#include "svg.h"

using namespace std;

const auto IMAGE_WIDTH = 300;
const auto IMAGE_HEIGHT = 400;
const auto TEXT_LEFT = 10;
const auto TEXT_BASELINE = 20;
const auto TEXT_WIDTH = 50;
const auto BIN_HEIGHT = 30;
const auto BLOCK_WIDTH = 10;

void
svg_begin(double width, double height) {
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg ";
    cout << "width='" << width << "' ";
    cout << "height='" << height << "' ";
    cout << "viewBox='0 0 " << width << " " << height << "' ";
    cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void
svg_end() {
    cout << "</svg>\n";
}

void
svg_text(double left, double baseline, string text) {
    cout << "<text x='" << left << "' y='" << baseline<< "'>" << text << "</text>";
}

void svg_rect(double x, double y, double width, double height, string stroke = "black", string fill = "black"){
    cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height << "' stroke='" << stroke << "' fill='" << fill << "' />";
}


void
show_histogram_svg(const vector<size_t>& bins) {
    double max = 0;
    int maxlen = IMAGE_WIDTH - TEXT_WIDTH;
    for (auto el: bins) {
        if (max < el) {
            max = el;
        }
    }
    double scale = max * BLOCK_WIDTH / maxlen;

    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);
    string stroke[] = {"red", "green", "blue"};
    string fill[] = {"red", "green", "blue"};
    int i = 0;
    double top = 0;
    for (size_t bin : bins) {
        const double bin_width = BLOCK_WIDTH * bin;
        double bin_height = bin_width / scale;
        svg_text(TEXT_LEFT + top, maxlen + TEXT_BASELINE, to_string(bin));
        svg_rect(top, maxlen - bin_height, BIN_HEIGHT, bin_height, stroke[i], fill[i]);
        top += BIN_HEIGHT;
        i++;
    }
    svg_end();
}
